@@ -33,16 +33,16 @@ from ._version import __version__
 from .miscellaneous import Configuration, StateManager, Heartbeat, start_application, write_ssh_config_file
 from .bootstrapagent import BootstrapAgent
 from .tunnel import Tunnel
-from .instructor import Instructor, TorInstructor, ForInstructor, FileInstructor, InteractiveInstructor
+from .instructor import Instructor, TorInstructor, ForInstructor, PlainInstructor
 from .transferagent import TransferAgent
 
-__author__ = '''Vincent Schouten <inquiry@intoreflection.co>'''
+__author__ = '''Vincent Schouten <powermole@protonmail.com>'''
 __docformat__ = '''google'''
 __date__ = '''06-04-2019'''
 __copyright__ = '''Copyright 2019, Vincent Schouten'''
 __license__ = '''MIT'''
 __maintainer__ = '''Vincent Schouten'''
-__email__ = '''<inquiry@intoreflection.co>'''
+__email__ = '''<powermole@protonmail.com>'''
 __status__ = '''Development'''  # "Prototype", "Development", "Production".
 
 # This is to 'use' the module(s), so lint doesn't complain
@@ -58,6 +58,5 @@ assert Instructor
 assert TransferAgent
 assert TorInstructor
 assert ForInstructor
-assert FileInstructor
-assert InteractiveInstructor
+assert PlainInstructor
 assert Tunnel
