@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 # File: powermolelibexceptions.py
 #
-# Copyright 2020 Vincent Schouten
+# Copyright 2021 Vincent Schouten
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to
@@ -31,21 +31,24 @@ Custom exception code for minitorcorelib.
 
 """
 
-__author__ = '''Vincent Schouten <inquiry@intoreflection.co>'''
+__author__ = '''Vincent Schouten <powermole@protonmail.com>'''
 __docformat__ = '''google'''
 __date__ = '''06-04-2019'''
-__copyright__ = '''Copyright 2019, Vincent Schouten'''
+__copyright__ = '''Copyright 2021, Vincent Schouten'''
 __credits__ = ["Vincent Schouten"]
 __license__ = '''MIT'''
 __maintainer__ = '''Vincent Schouten'''
-__email__ = '''<inquiry@intoreflection.co>'''
+__email__ = '''<powermole@protonmail.com>'''
 __status__ = '''Development'''  # "Prototype", "Development", "Production".
 
 
 class InvalidConfigurationFile(Exception):
     """The configuration file provided is not valid."""
-    #
-    # def __init__(self):
-    #     self.message = 'configuration invalid'
-    #     # Call the base class constructor with the parameters it needs
-    #     super().__init__(self.message)
+
+
+class TransferError(Exception):
+    """Something went wrong during transfer of the file."""
+
+
+class InvalidDataStructure(Exception):
+    """The data structure is invalid."""
